//  Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.

enum AGConnectCloudDBExceptionType {
  /// Some resources have been exhausted. The number of resources exceeds
  /// the upper limit or the system resources are exhausted.
  RESOURCE_EXHAUSTED,

  /// Network connection timed out.
  TIMEOUT,

  /// The operation is rejected. The system does not support the current operation.
  /// Rectify the fault based on the error information.
  ///
  /// * For example, deleting the Cloud DB zone before closing it will be rejected.
  ///   Close the Cloud DB zone first.
  FAILED_PRECONDITION,

  /// The user does not have permissions to perform this operation.
  PERMISSION_DENIED,

  /// An unknown exception or an exception that does not belong to the AGConnectCloudDBPlugin occurs.
  UNKNOWN,

  /// The data size is too large.
  ///
  /// * For example, when data is queried on the cloud,
  ///   the size of the returned result set exceeds the limit.
  DATA_SIZE_OVERFLOW,

  /// The entered user password is invalid.
  ///
  /// * For example, the password does not meet
  ///   the password requirements or the password is incorrect.
  USER_KEY_ILLEGALITY,

  /// The data key on the cloud has been changed but is not updated to the device.
  /// The data encrypted using the original key on the device
  /// cannot be synchronized to the cloud. The key on the device
  /// needs to be updated to the latest version.
  DATA_ENCRYPTION_KEY_CHANGED,

  /// Cloud DB Zone not found, first try to open it.
  ZONE_NOT_FOUND,
}
