//  Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import '../enums/exception_type.dart';

class AGConnectCloudDBException implements Exception {
  final String code;
  final String message;
  final String _type;

  const AGConnectCloudDBException._internal({
    @required this.code,
    @required this.message,
    @required String type,
  }) : _type = type;

  factory AGConnectCloudDBException.from(dynamic e) {
    if (e is AGConnectCloudDBException) {
      return e;
    } else {
      if (e is PlatformException) {
        return AGConnectCloudDBException._internal(
          code: e.code,
          message: e.message ?? 'Something went wrong.',
          type: Map<String, dynamic>.from(e.details ?? <String, dynamic>{})['exceptionType'] ?? '',
        );
      } else {
        return AGConnectCloudDBException._internal(
          code: 'UNKNOWN',
          message: e is String ? e : 'Something went wrong.',
          type: 'PluginException',
        );
      }
    }
  }

  AGConnectCloudDBExceptionType get type {
    if (_type == 'PluginException') {
      switch (code) {
        case 'UNKNOWN':
          return AGConnectCloudDBExceptionType.UNKNOWN;
        case 'ZONE_NOT_FOUND':
          return AGConnectCloudDBExceptionType.ZONE_NOT_FOUND;
      }
    } else if (_type == 'SDKException') {
      switch (code) {
        case '1':
          return AGConnectCloudDBExceptionType.RESOURCE_EXHAUSTED;
        case '2':
          return AGConnectCloudDBExceptionType.TIMEOUT;
        case '3':
          return AGConnectCloudDBExceptionType.FAILED_PRECONDITION;
        case '4':
          return AGConnectCloudDBExceptionType.PERMISSION_DENIED;
        case '5':
          return AGConnectCloudDBExceptionType.UNKNOWN;
        case '6':
          return AGConnectCloudDBExceptionType.DATA_SIZE_OVERFLOW;
        case '7':
          return AGConnectCloudDBExceptionType.USER_KEY_ILLEGALITY;
        case '8':
          return AGConnectCloudDBExceptionType.DATA_ENCRYPTION_KEY_CHANGED;
      }
    }
    return AGConnectCloudDBExceptionType.UNKNOWN;
  }

  @override
  String toString() {
    return 'AGConnectCloudDBException(code: $code message: $message type:$type)';
  }
}
