//  Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.

import 'package:flutter/foundation.dart';

import '../enums/zone_access_property.dart';
import '../enums/zone_sync_property.dart';

class AGConnectCloudDBZoneConfig {
  String _zoneName;
  AGConnectCloudDBZoneSyncProperty _syncProperty;
  AGConnectCloudDBZoneAccessProperty _accessProperty;
  int _capacity;
  bool _isPersistenceEnabled;
  String _key;
  String _reKey;
  bool _isEncrypted;

  String get zoneName => _zoneName;
  AGConnectCloudDBZoneSyncProperty get syncProperty => _syncProperty;
  AGConnectCloudDBZoneAccessProperty get accessProperty => _accessProperty;
  int get capacity => _capacity;
  bool get isPersistenceEnabled => _isPersistenceEnabled;
  bool get isEncrypted => _isEncrypted;

  AGConnectCloudDBZoneConfig({
    @required String zoneName,
    AGConnectCloudDBZoneSyncProperty syncProperty = AGConnectCloudDBZoneSyncProperty.CLOUDDBZONE_CLOUD_CACHE,
    AGConnectCloudDBZoneAccessProperty accessProperty = AGConnectCloudDBZoneAccessProperty.CLOUDDBZONE_PUBLIC,
    int capacity = 104857600,
    bool isPersistenceEnabled = true,
    String key = '',
    String reKey = '',
  }) {
    if (zoneName == null || zoneName.isEmpty) {
      throw FormatException('Value of zoneName cannot be null or an empty string.', zoneName);
    }
    if (syncProperty == null) {
      throw FormatException('Value of syncProperty cannot be null.', syncProperty);
    }
    if (accessProperty == null) {
      throw FormatException('Value of accessProperty cannot be null.', accessProperty);
    }
    if (capacity == null || capacity <= 0) {
      throw FormatException('Value of capacity cannot be null or a negative number.', capacity);
    }
    if (isPersistenceEnabled == null) {
      throw FormatException('Value of isPersistenceEnabled cannot be null.', isPersistenceEnabled);
    }
    if (key == null || reKey == null) {
      throw FormatException('Value of key and reKey cannot be null.');
    } else {
      if (key.isEmpty && reKey.isNotEmpty) {
        throw FormatException('Value of key cannot be empty string when reKey has a value.');
      }
    }
    this._zoneName = zoneName;
    this._syncProperty = syncProperty;
    this._accessProperty = accessProperty;
    this._capacity = capacity;
    this._isPersistenceEnabled = isPersistenceEnabled;
    this._zoneName = zoneName;
    this._key = key;
    this._reKey = reKey;
    this._isEncrypted = key.isNotEmpty;
  }

  factory AGConnectCloudDBZoneConfig.fromMap(Map<String, dynamic> map) {
    if (map == null) return null;

    final AGConnectCloudDBZoneConfig config = AGConnectCloudDBZoneConfig(
      zoneName: map['zoneName'] == null ? null : map['zoneName'],
      syncProperty: map['syncProperty'] == null
          ? null
          : AGConnectCloudDBZoneSyncProperty.values.firstWhere(
              (AGConnectCloudDBZoneSyncProperty e) => describeEnum(e) == map['syncProperty'],
              orElse: () => null,
            ),
      accessProperty: map['accessProperty'] == null
          ? null
          : AGConnectCloudDBZoneAccessProperty.values.firstWhere(
              (AGConnectCloudDBZoneAccessProperty e) => describeEnum(e) == map['accessProperty'],
              orElse: () => null,
            ),
      capacity: map['capacity'] == null ? null : map['capacity'],
      isPersistenceEnabled: map['isPersistenceEnabled'] == null ? null : map['isPersistenceEnabled'],
    );
    config._isEncrypted = map['isEncrypted'] ?? false;
    return config;
  }

  Map<String, dynamic> toMap() {
    return <String, dynamic>{
      'zoneName': zoneName,
      'syncProperty': syncProperty.index,
      'accessProperty': accessProperty.index,
      'capacity': capacity,
      'isPersistenceEnabled': isPersistenceEnabled,
      'userKey': _key,
      'userReKey': _reKey,
      'isEncrypted': _isEncrypted,
    };
  }

  @override
  String toString() {
    return 'AGConnectCloudDBZoneConfig ' +
        'zoneName: $zoneName ' +
        'isEncrypted: $isEncrypted ' +
        'isPersistenceEnabled: $isPersistenceEnabled ' +
        'capacity: $capacity ' +
        'syncProperty: ${describeEnum(syncProperty)} ' +
        'accessProperty: ${describeEnum(accessProperty)}';
  }
}
